@@ -39,6 +39,9 @@ class Surface(object):
     The ``width`` and ``height`` attributes are in device units (pixels for
     PNG, else points).
 
+    The ``context_width`` and ``context_height`` attributes are in user units
+    (i.e. in pixels), they represent the size of the active viewport.
+
     """
 
     # Subclasses must either define this or override _create_surface()
@@ -88,6 +91,8 @@ class Surface(object):
         """
         self.cairo = None
         self.context = None
+        self.context_width, self.context_height = None, None
+        self.font_size = None
         self.cursor_position = 0, 0
         self.total_width = 0
         self.markers = {}
@@ -137,9 +142,10 @@ class Surface(object):
         return cairo_surface, width, height
 
     def set_context_size(self, width, height, viewbox):
-        """Set the context size."""
+        """Set the Cairo context size, set the SVG viewport size."""
         if viewbox:
             x, y, x_size, y_size = viewbox
+            self.context_width, self.context_height = x_size, y_size
             x_ratio, y_ratio = width / x_size, height / y_size
             if x_ratio > y_ratio:
                 self.context.translate((width - x_size * y_ratio) / 2, 0)
@@ -152,6 +158,8 @@ class Surface(object):
             else:
                 self.context.scale(x_ratio, y_ratio)
                 self.context.translate(-x, -y)
+        else:
+            self.context_width, self.context_height = width, height
 
     def finish(self):
         """Read the surface content."""
@@ -163,21 +171,34 @@ class Surface(object):
 
     def draw(self, node, stroke_and_fill=True):
         """Draw ``node`` and its children."""
+        old_font_size = self.font_size
+        self.font_size = size(self, node.get("font-size", "12pt"))
+
         # Do not draw defs
         if node.tag == "defs":
             for child in node.children:
                 parse_def(self, child)
             return
 
+        # Do not draw elements with width or height of 0
+        if (("width" in node and size(self, node["width"]) == 0) or
+            ("height" in node and size(self, node["height"]) == 0)):
+            return
+
         node.tangents = [None]
         node.pending_markers = []
 
         self._old_parent_node = self.parent_node
         self.parent_node = node
 
+        opacity = float(node.get("opacity", 1))
+        if opacity < 1:
+            self.context.push_group()
+
         self.context.save()
         self.context.move_to(
-            size(self, node.get("x")), size(self, node.get("y")))
+            size(self, node.get("x"), "x"),
+            size(self, node.get("y"), "y"))
 
         # Transform the context according to the ``transform`` attribute
         transform(self, node.get("transform"))
@@ -207,9 +228,8 @@ class Surface(object):
             TAGS[node.tag](self, node)
 
         # Get stroke and fill opacity
-        opacity = float(node.get("opacity", 1))
-        stroke_opacity = opacity * float(node.get("stroke-opacity", 1))
-        fill_opacity = opacity * float(node.get("fill-opacity", 1))
+        stroke_opacity = float(node.get("stroke-opacity", 1))
+        fill_opacity = float(node.get("fill-opacity", 1))
 
         # Manage dispaly and visibility
         display = node.get("display", "inline") != "none"
@@ -242,7 +262,12 @@ class Surface(object):
             # raise an exception if we have multiple svg tags
             self.context.restore()
 
+        if opacity < 1:
+            self.context.pop_group_to_source()
+            self.context.paint_with_alpha(opacity)
+
         self.parent_node = self._old_parent_node
+        self.font_size = old_font_size
 
 
 class MultipageSurface(Surface):
@@ -255,13 +280,12 @@ class MultipageSurface(Surface):
             # Multi-page
             for page in svg_children:
                 width, height, viewbox = node_format(self, page)
-                # TODO: test this
+                self.context.save()
+                self.set_context_size(width, height, viewbox)
                 width *= self.device_units_per_user_units
                 height *= self.device_units_per_user_units
                 self.page_sizes.append((width, height))
                 self.cairo.set_size(width, height)
-                self.context.save()
-                self.set_context_size(width, height, viewbox)
                 self.draw(page)
                 self.context.restore()
                 self.cairo.show_page()
