@@ -57,10 +57,10 @@ def gradient(surface, node):
     """Gradients colors."""
     gradient_node = surface.gradients[filter_fill_content(node)]
 
-    x = float(size(surface, node.get("x")))
-    y = float(size(surface, node.get("y")))
-    height = float(size(surface, node.get("height")))
-    width = float(size(surface, node.get("width")))
+    x = float(size(surface, node.get("x"), "x"))
+    y = float(size(surface, node.get("y"), "y"))
+    height = float(size(surface, node.get("height"), "x"))
+    width = float(size(surface, node.get("width"), "y"))
     x1 = float(gradient_node.get("x1", x))
     x2 = float(gradient_node.get("x2", x + width))
     y1 = float(gradient_node.get("y1", y))
@@ -73,10 +73,8 @@ def gradient(surface, node):
         for child in gradient_node.children:
             stop_color = color(
                 child.get("stop-color"), float(child.get("stop-opacity", 1)))
-            offset = child.get("offset")
-            if "%" in offset:
-                offset = float(offset.strip("%")) / 100
-            linpat.add_color_stop_rgba(float(offset), *stop_color)
+            offset = size(surface, child.get("offset"), 1)
+            linpat.add_color_stop_rgba(offset, *stop_color)
         surface.context.set_source(linpat)
     elif gradient_node.tag == "radialGradient":
         r = float(gradient_node.get("r"))
@@ -87,9 +85,7 @@ def gradient(surface, node):
         radpat = cairo.RadialGradient(fx, fy, 0, cx, cy, r)
 
         for child in gradient_node.children:
-            offset = child.get("offset")
-            if "%" in offset:
-                offset = float(offset.strip("%")) / 100
+            offset = size(surface, child.get("offset"), 1)
             stop_color = color(
                 child.get("stop-color"), child.get("stop-opacity", 1))
             radpat.add_color_stop_rgba(float(offset), *stop_color)
@@ -113,15 +109,13 @@ def pattern(surface, node):
     pattern_node = surface.patterns[filter_fill_content(node)]
     transform(surface, pattern_node.get("patternTransform"))
 
-    if pattern_node.tag == "pattern":
-        from . import SVGSurface  # circular import
-        pattern_surface = SVGSurface(
-            tree=pattern_node, output=None, dpi=surface.dpi)
-        pattern_pattern = cairo.SurfacePattern(pattern_surface.cairo)
-        pattern_pattern.set_extend(cairo.EXTEND_REPEAT)
-        surface.context.set_source(pattern_pattern)
-        surface.context.fill_preserve()
-        pattern_surface.finish()
+    from . import SVGSurface  # circular import
+    pattern_surface = SVGSurface(pattern_node, None, surface.dpi)
+    pattern_pattern = cairo.SurfacePattern(pattern_surface.cairo)
+    pattern_pattern.set_extend(cairo.EXTEND_REPEAT)
+    surface.context.set_source(pattern_pattern)
+    surface.context.fill_preserve()
+    pattern_surface.finish()
 
 
 def draw_marker(surface, node, position="mid"):
@@ -146,11 +140,8 @@ def draw_marker(surface, node, position="mid"):
 
     while node.pending_markers:
         next_point, markers = node.pending_markers.pop(0)
-
-        if node.tangents != []:
-            angle1 = node.tangents.pop(0)
-        if node.tangents != []:
-            angle2 = node.tangents.pop(0)
+        angle1 = node.tangents.pop(0)
+        angle2 = node.tangents.pop(0)
 
         for active_marker in markers:
             if not active_marker.startswith("#"):
@@ -161,8 +152,6 @@ def draw_marker(surface, node, position="mid"):
 
                 angle = marker_node.get("orient", "0")
                 if angle == "auto":
-                    if angle1 is None:
-                        angle1 = angle2
                     angle = float(angle1 + angle2) / 2
                 else:
                     angle = radians(float(angle))
@@ -210,7 +199,7 @@ def use(surface, node):
     """Draw the content of another SVG file."""
     surface.context.save()
     surface.context.translate(
-        size(surface, node.get("x")), size(surface, node.get("y")))
+        size(surface, node.get("x"), "x"), size(surface, node.get("y"), "y"))
     if "x" in node:
         del node["x"]
     if "y" in node:
