@@ -99,21 +99,18 @@ def text(surface, node):
     text_extents = surface.context.text_extents(node.text)
     x_bearing = text_extents[0]
     width = text_extents[2]
-    x, y = size(surface, node.get("x")), size(surface, node.get("y"))
+    x, y = size(surface, node.get("x"), "x"), size(surface, node.get("y"), "y")
     text_anchor = node.get("text-anchor")
     if text_anchor == "middle":
         x -= width / 2. + x_bearing
     elif text_anchor == "end":
         x -= width + x_bearing
 
-    # Get global text opacity
-    opacity = float(node.get("opacity", 1))
-
     surface.context.move_to(x, y)
     if "url(#" in node.get("fill"):
         gradient_or_pattern(surface, node)
     else:
-        surface.context.set_source_rgba(*color(node.get("fill"), opacity))
+        surface.context.set_source_rgba(*color(node.get("fill")))
     surface.context.show_text(node.text)
     node["fill"] = "#00000000"
 
@@ -124,9 +121,8 @@ def text(surface, node):
 def text_path(surface, node):
     """Draw text on a path."""
     surface.context.save()
-    opacity = float(node.get("opacity", 1))
     if "url(#" not in node.get("fill"):
-        surface.context.set_source_rgba(*color(node.get("fill"), opacity))
+        surface.context.set_source_rgba(*color(node.get("fill")))
 
     id_path = node.get("{http://www.w3.org/1999/xlink}href")
     if not id_path.startswith("#"):
@@ -142,14 +138,8 @@ def text_path(surface, node):
     cairo_path = surface.context.copy_path_flat()
     surface.context.new_path()
 
-    start_offset = node.get("startOffset", 0)
-    if start_offset:
-        if "%" in start_offset:
-            start_offset = (
-                path_length(cairo_path) *
-                float(start_offset.rstrip(" %")) / 100)
-        else:
-            start_offset = size(surface, start_offset)
+    start_offset = size(
+        surface, node.get("startOffset", 0), path_length(cairo_path))
     surface.total_width += start_offset
 
     x, y = point_following_path(cairo_path, surface.total_width)
@@ -168,7 +158,7 @@ def text_path(surface, node):
         surface.context.save()
         surface.context.translate(x, y)
         surface.context.rotate(angle)
-        surface.context.translate(0, size(surface, node.get("y")))
+        surface.context.translate(0, size(surface, node.get("y"), "y"))
         surface.context.move_to(0, 0)
         surface.context.show_text(letter)
         surface.context.restore()
@@ -177,18 +167,20 @@ def text_path(surface, node):
 
     # Remember the relative cursor position
     surface.cursor_position = \
-        size(surface, node.get("x")), size(surface, node.get("y"))
+        size(surface, node.get("x"), "x"), size(surface, node.get("y"), "y")
 
 
 def tspan(surface, node):
     """Draw a tspan ``node``."""
     x, y = [[i] for i in surface.cursor_position]
     if "x" in node:
-        x = [size(surface, i) for i in normalize(node["x"]).strip().split(" ")]
+        x = [size(surface, i, "x")
+             for i in normalize(node["x"]).strip().split(" ")]
     if "y" in node:
-        y = [size(surface, i) for i in normalize(node["y"]).strip().split(" ")]
+        y = [size(surface, i, "y")
+             for i in normalize(node["y"]).strip().split(" ")]
 
-    string = (node.text or '').strip()
+    string = (node.text or "").strip()
     if not string:
         return
     fill = node.get("fill")
@@ -202,15 +194,15 @@ def tspan(surface, node):
             x = surface.cursor_position[0]
         if y == None:
             y = surface.cursor_position[1]
-        node["x"] = str(x + size(surface, node.get("dx")))
-        node["y"] = str(y + size(surface, node.get("dy")))
+        node["x"] = str(x + size(surface, node.get("dx"), "x"))
+        node["y"] = str(y + size(surface, node.get("dy"), "y"))
         node["fill"] = fill
         node.text = letters
         if node.parent.tag == "text":
             text(surface, node)
         else:
-            node["x"] = str(x + size(surface, node.get("dx")))
-            node["y"] = str(y + size(surface, node.get("dy")))
+            node["x"] = str(x + size(surface, node.get("dx"), "x"))
+            node["y"] = str(y + size(surface, node.get("dy"), "y"))
             text_path(surface, node)
             if node.parent.children[-1] == node:
                 surface.total_width = 0
